#!/usr/bin/ruby

# Usage:
#
# * install a fence agent package e.g. fence-agents-ilo2
# * fence_ilo2 -o metadata > ilo.xml
# * fence-generator.rb ilo.xml fence_ilo2 fence-agents-ilo2
#      [ XML metadata, name of the class, name of the package for dependency check ]


require 'rexml/document'

class FencingMetadataParser
  def initialize(filename, agentName, packageName)
    @agentName = agentName
    @packageName = packageName
    file = File.new(filename)
    @doc = REXML::Document.new file
  end

  def getPackageName()
    return @packageName
  end

  def getAgentName()
    return @agentName
  end

  def getParameters()
    ## result have to be array as order should be preserved
    params = Array.new

    @doc.elements.each("resource-agent/parameters/parameter") { |p|
      param = Hash.new
      param["name"] = REXML::XPath.match(p, "string(./@name)")[0]
      param["type"] = REXML::XPath.match(p, "string(./content/@type)")[0]
      ## if 'default' is list then we can not enter it as parameter !!
      ## this is problem only for 'cmd_prompt'
      param["default"] = REXML::XPath.match(p, "string(./content/@default)")[0]

      ## remove parameters that are not usable during automatic execution
      if not ["help", "version", "action"].include?(param["name"])
        params.push(param)
      end
    }
    return params
  end
end

class ManifestGenerator
  def initialize(parser)
    @parser = parser
  end

  def generate
    puts <<-eos
# generated by agent_generator.rb, manual changes will be lost

define pacemaker::stonith::#{@parser.getAgentName} (
#{getManifestParameters}
) {
#{getVariableValues}
  $pcmk_host_value_chunk = $pcmk_host_list ? {
    undef => '$(/usr/sbin/crm_node -n)',
    default => "${pcmk_host_list}",
  }

  # $title can be a mac address, remove the colons for pcmk resource name
  $safe_title = regsubst($title, ':', '', 'G')

  if($ensure == absent) {
    exec { "Delete stonith-#{@parser.getAgentName}-${safe_title}":
      command => "/usr/sbin/pcs stonith delete stonith-#{@parser.getAgentName}-${safe_title}",
      onlyif => "/usr/sbin/pcs stonith show stonith-#{@parser.getAgentName}-${safe_title} > /dev/null 2>&1",
      require => Class["pacemaker::corosync"],
    }
  } else {
    package {
      "#{@parser.getPackageName}": ensure => installed,
    } ->
    exec { "Create stonith-#{@parser.getAgentName}-${safe_title}":
      command => "/usr/sbin/pcs stonith create stonith-#{@parser.getAgentName}-${safe_title} #{@parser.getAgentName} pcmk_host_list=\\"${pcmk_host_value_chunk}\\" #{getChunks} op monitor interval=${interval}",
      unless => "/usr/sbin/pcs stonith show stonith-#{@parser.getAgentName}-${safe_title} > /dev/null 2>&1",
      tries => $tries,
      try_sleep => $try_sleep,
      require => Class["pacemaker::corosync"],
    } ->
    exec { "Add non-local constraint for stonith-#{@parser.getAgentName}-${safe_title}":
      command => "/usr/sbin/pcs constraint location stonith-#{@parser.getAgentName}-${safe_title} avoids ${pcmk_host_value_chunk}",
      tries => $tries,
      try_sleep => $try_sleep,
    }
  }
}
eos
  end

  def getManifestParameters
    text = ""
    @parser.getParameters.each { |p|
      text += "  $#{p['name']} = undef,\n"
    }

    text += "\n"
    text += "  $interval = \"60s\",\n"
    text += "  $ensure = present,\n"
    text += "  $pcmk_host_list = undef,\n"
    text += "\n"
    text += "  $tries = undef,\n"
    text += "  $try_sleep = undef,"

    return text
  end

  def getVariableValues
    text = ""
    @parser.getParameters.each { |p|
      text += "  $#{p['name']}_chunk = $#{p['name']} ? {\n"
      text += "    undef => \"\",\n"
      text += "    default => \"#{p['name']}=\\\"${#{p['name']}}\\\"\",\n"
      text += "  }\n"
    }

    return text
  end

  def getChunks
    text = ""
    @parser.getParameters.each { |p|
      text += "${#{p['name']}_chunk} "
    }
    return text
  end
end

if ARGV.length != 3 then
  puts "You have to enter three arguments: path to metadata, name of fence agent and fence agent package"
  exit 1
end

metadata, agentName, packageName = ARGV
# e.g. parser = FencingMetadataParser.new("ilo.xml", "fence_ilo", "fence-agents-ilo2")
parser = FencingMetadataParser.new(metadata, agentName, packageName)
ManifestGenerator.new(parser).generate
